local name, addon = ...
local parentName = addon['parentName'] or "BrokerPack"
local childName = "zz_Coords"
local gt_r, gt_g, gt_b, gt_a

local defaults = {
	["colorLDB"] = true,
	["ShowZone"] = false,
	["ShowSubZone"] = true,
	["ShowTooltip"] = false,
}
local options = {
}
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local ldbicon = LibStub("LibDBIcon-1.0")
local mapFrame
local playerCoords
local OFFSET_X = 0.0022
local OFFSET_Y = -0.0262
local zoneText,subZoneText
local coordsline,col
local L = LibStub("AceLocale-3.0"):GetLocale(childName, true)
local outbound = true
local mf, mfn
local db
local delayMapUpdate = .2
local delayOnUpdate = .5

local function isoutbound()
	mf = GetMouseFocus()
	if(mf) then
		mfn = GetMouseFocus():GetName()
		if(mfn and mfn ~= "WorldMapFrame") then
			if((string.sub(mfn,1,8) == "WorldMap") or (string.sub(mfn,1,8) == "poiWorld"))then return false end
		end
	end
	return true
end

local function positionFrame()
	if(WORLDMAP_SETTINGS['size'] ~= WORLDMAP_WINDOWED_SIZE) then
		zz_CoordsMapFrameCoordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", db['screenX'], db['screenY'])
	else
		zz_CoordsMapFrameCoordText:SetText()
		zz_CoordsMapFrameCoordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", db['windowX'], db['windowY'])
	end
end
local function limitLength(info, maxlength)
	if(info and maxlength) then
		if(maxlength > 0 and info:len() > maxlength) then
			info = info:sub(1,maxlength).."~"
		end
	end
	return info
end

local function setPlayerCoords()
  local x,y = GetPlayerMapPosition("player")
  playerCoords = format("%.1f, %.1f", x*100, y*100)
end

local function pvpColor(txt)
	local pvpType, isFFA, faction = GetZonePVPInfo()
	if(pvpType == "friendly") then
		col = "00ff00"
	elseif(pvpType == "hostile") then
		col = "ff0000"
	elseif(pvpType == "arena") then
		col = "ffff00"
	elseif(pvpType == "hostile" or pvpType == "combat" or pvpType=="contested") then
		if(faction == select(2,UnitFactionGroup('player'))) then
			col = "886600"
		else
			col = "ff0000"
		end
	elseif(pvpType == "sanctuary") then
		col = "9999ff"
	else
		return txt
	end
	return addon['colorize'](txt, col)
end

local function OnText(message)
	addon['OnText'](childName, message)
end

local function OnTooltip(tip)
	local p = tip:GetParent()
	if(p.tooltip) then
		tip = p.tooltip
	end
	local n = type(tip.AddHeader) == 'function' and tip:AddHeader(childName) or tip:AddLine(childName)
	tip:AddLine(GetRealZoneText())
	tip:AddLine(GetSubZoneText())
	tip:AddLine(format("%s: %s",childName,tostring(playerCoords)))
	if(info~="inbound") then
		tip:AddLine(L['HINT'])
		tip:AddLine(L['HINT2'])
		tip:AddLine(L['HINT3'])
		tip:AddLine(L['HINT4'])
	end
end

local function OnButton(self, button)
	if(button == 'RightButton') then
		if(IsShiftKeyDown()) then
			zzCoords_MinimapToggle()
		else
			InterfaceOptionsFrame_OpenToCategory(childName)
		end
	elseif(button == "LeftButton") then
		if(IsShiftKeyDown()) then
			local db = addon['db']['global']['ldbicons']
			db[childName]['hide'] = not db[childName]['hide']
			if(ldbicon) then
				if(db[childName]['hide']) then
					ldbicon:Hide(childName)
				else
					ldbicon:Show(childName)
				end
			end
		else
			ToggleFrame(WorldMapFrame)
		end
	end
end

local function MouseOut(self)
	addon['qtip']:Release(self['tooltip'])
	self['tooltip'] = nil
end

local function MouseIn(self,adjustedX, adjustedY,playerCoords)
	if(db['hideMapCoords']) then return end
	local tip = addon['qtip']:Acquire(childName.."QTip",2)
	tip:SetParent(WorldMapFrame)
	tip:SetPoint("BOTTOMRIGHT",WorldMapFrame,"BOTTOMRIGHT",-30,0)
	tip:Clear()
	self['tooltip'] = tip
	tip:AddLine(L["Cursor"],format("%.1f, %.1f",adjustedX, adjustedY))
	tip:AddLine(L["Player"],playerCoords)
	tip:SetBackdropColor(gt_r, gt_g, gt_b, .5)
	tip:Show()
end

local function OnDataUpdate()
	setPlayerCoords()
	if(zoneText and zoneText~="") then
		if(db['hideLDBCoords']) then
			coordsline = "%s"
		else
			coordsline = "%s (%s)"
		end
		OnText(format(coordsline,db['colorLDB'] and pvpColor(zoneText) or zoneText,playerCoords))
	else
		if(db['hideLDBCoords']) then
			OnText("")
		else
			OnText(playerCoords)
		end
	end
	C_Timer.After(delayOnUpdate, OnDataUpdate)
end

local function ZONED()
	if(not WorldMapFrame:IsShown()) then
		SetMapToCurrentZone()
	end
	zoneText = db['ShowZone'] and GetRealZoneText() or ""
	if(db['ShowSubZone']) then
		if(GetSubZoneText() and GetSubZoneText()~="") then
			if(db['ShowZone']) then
				zoneText = zoneText ~= GetSubZoneText() and format("%s, %s",zoneText,GetSubZoneText()) or zoneText
			else
				zoneText = GetSubZoneText()
			end
		elseif(not db['ShowZone']) then
			zoneText = GetRealZoneText()
		end
	end
	zoneText = limitLength(zoneText,db['maxLength'])
end

local function MapFrame_OnUpdate()
	C_Timer.After(delayMapUpdate, MapFrame_OnUpdate)
	if(not zz_CoordsMapFrameCoordText:IsVisible()) then return end
	setPlayerCoords()
	local x, y = GetCursorPosition();
	x = x / WorldMapButton:GetEffectiveScale();
	y = y / WorldMapButton:GetEffectiveScale();

	local centerX, centerY = WorldMapButton:GetCenter();
	local width = WorldMapButton:GetWidth();
	local height = WorldMapButton:GetHeight();
	local adjustedY = addon['round']((centerY + (height/2) - y) / height * 100,1);
	local adjustedX = addon['round']((x - (centerX - (width/2))) / width * 100,1);
	zz_CoordsMapFrameCoordText:SetText(format("%s%s",isoutbound() and L['outbound'] or L['COORDFMTSTRING']( mapFrame,adjustedX, adjustedY), playerCoords))
	positionFrame()
	if(isoutbound()) then
		MouseOut(mapFrame)
	else
		MouseIn(mapFrame,adjustedX, adjustedY,playerCoords)
	end
end

local function mainFrame()
	mapFrame = CreateFrame("Frame", "zz_CoordsMapFrame", WorldMapFrame)
	C_Timer.After(delayMapUpdate, MapFrame_OnUpdate)
	local coordText = mapFrame:CreateFontString("zz_CoordsMapFrameCoordText", "ARTWORK")
	coordText:SetFont(GameFontNormal:GetFont())
	coordText:SetTextColor(GameFontNormal:GetTextColor())
	coordText:SetShadowColor(GameFontNormal:GetShadowColor())
	coordText:SetShadowOffset(GameFontNormal:GetShadowOffset())
	coordText:SetPoint("BOTTOM", WorldMapFrame, "BOTTOM", 0, 10)
end

local function getPref(pref)
	return db[pref[#pref]]
end

local function setPref(pref,value)
	db[pref[#pref]] = value
	ZONED()
end

local function init()
	options = addon['InitConfig'](addon, childName, true, {
		['type'] = "data source",
		['OnTooltipShow'] = OnTooltip,
		['OnClick'] = OnButton,
	}, getPref, setPref)
	db = addon['db']['profile'][childName]
	mainFrame()
	C_Timer.After(delayOnUpdate, OnDataUpdate)
	BINDING_HEADER_COORDS = childName
	BINDING_NAME_COORDS_TOGGLE = L["Toggle Minimap"]
	
	addon['AddConfigEntry'](addon, childName,"toggle","hideLDBCoords",L['Hide LDB coords'],L['This hides the coords-display in your broker display'],2,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"toggle","colorLDB",L['Colorized Zone'],L['Show PvP informations about current area'],1,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"toggle","hideMapCoords",L['Hide Map coords'],L['This hides the coords-display at the bottom of your map'],2,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"toggle","ShowZone",L['Show Zone'],nil,1,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"toggle","ShowSubZone",L['Show Subzone'],nil,1,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"range","maxLength",L['Max Length'],L['Limits the maximum length of the zone'],1,0,255,1,false,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"header",'screenXY',L['Mapcoords Screen'],nil,3,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"header",'windowXY',L['Mapcoords Window'],nil,6,nil,nil,nil,nil,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"range","screenX",L['Screen X'],L['Adjusts the X-Offset on fullscreenmap'],4,-1024,1024,1,false,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"range","screenY",L['Screen Y'],L['Adjusts the Y-Offset on fullscreenmap'],5,-1024,1024,1,false,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"range","windowX",L['Window X'],L['Adjusts the X-Offset on windowed map'],7,-512,512,1,false,options['args'][childName])
	addon['AddConfigEntry'](addon, childName,"range","windowY",L['Window Y'],L['Adjusts the Y-Offset on windowed map'],8,-512,512,1,false,options['args'][childName])
	if(not db['hideMapCoords']) then
		zz_CoordsMapFrame:Show()
	else
		zz_CoordsMapFrame:Hide()
	end
	db['screenX'] = db['screenX'] or 0
	db['screenY'] = db['screenY'] or 11
	db['windowX'] = db['windowX'] or -10
	db['windowY'] = db['windowY'] or -3
	addon['RegisterFunc']({"ZONE_CHANGED_NEW_AREA","ZONE_CHANGED","ZONE_CHANGED_INDOORS"},"OnEvent", ZONED)
	WorldMapFrame:HookScript("OnEnter", function(self) outbound = false end)
	WorldMapFrame:HookScript("OnLeave", function(self) outbound = true end)
	ZONED()
end

function zzCoords_MinimapToggle()
	if(MinimapCluster:IsVisible()) then
		MinimapCluster:Hide()
	else
		MinimapCluster:Show()
	end
end
addon['startup'](addon, name, childName, init, true, defaults)
